##################### PSQF 6243 Example 2 using R #############################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=5, scipen=9, show.signif.stars=FALSE)

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import excel XLSX data as table

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("supernova")) install.packages("supernova")
library(supernova) # To get sums of squares table

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("lm.beta")) install.packages("lm.beta")
library(lm.beta) # To do glht linear combinations in GLMs

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files using code below


################################################################################
#####            BEGIN EXAMPLE 2 DATA IMPORT AND MANIPULATION              #####
################################################################################

# Set working directory (to import and export files to)
# Paste in the folder address where "GSS_Example.xlsx" is saved in quotes
setwd("C:/Dropbox/24_PSQF6243/PSQF6243_Example2")

# Import GSS_Example.xlsx data from working directory -- path = file name
Example2 = read_excel(path="GSS_Example.xlsx", sheet="GSS_Example") 
# Convert to data frame to use for analysis
Example2 = as.data.frame(Example2)

# Label variables used below (add descriptive titles) using comments instead
#   because variable labels are not allowed in many analysis packages
# Format: name = Descriptive Variable Label
# marry  = marry: Marital Status (1=unmarried, 2=married)
# educ   = educ: Years of Education
# income = income: Annual Income in 1000s

# Example of sorting to find specific case for slides
Example2 = Example2[order(Example2$educ, Example2$income), ]

# Open external file to save results to
txtStart(file="Example2_R_Output.txt")


################################################################################
#####                BEGIN EXAMPLE 2 DATA DESCRIPTION                      #####
################################################################################

# describe prints sample descriptive statistics for quantitative variables
# list variables to be included in separate quotes within c concatenate function
# wrapped a print command around to get more than two significant digits
print("R Descriptive Statistics for Quantitative or Binary Variables")
print(describe(x=Example2[ , c("income","educ","marry")], fast=TRUE), digits=3)

# Get variances too (on diagonal of output matrix)
var(x=Example2[ , c("income","educ","marry")])

txtComment(" ") # insert blank space in external file of saved results


################################################################################
#####             BEGIN EXAMPLE 2 GENERAL LINEAR MODELS                    #####
################################################################################

print("R Empty GLM Predicting Income -- save as ModelEmpty")
ModelEmpty = lm(data=Example2, formula=income~1) # 1 represents intercept
supernova(ModelEmpty)   # supernova prints sums of squares and residual variance
summary(ModelEmpty)     # summary prints fixed effects solution 
confint(ModelEmpty, level=.95) # confint prints level% CI for fixed effects

txtComment(" ") # insert blank space in external file of saved results


####### MODELS WITH QUANTITATIVE EDUCATION AS A PREDICTOR #######

print("R GLM Predicting Income from Original Education -- save as ModelEduc")
ModelEduc = lm(data=Example2, formula=income~1+educ)
supernova(ModelEduc)    # supernova prints sums of squares and residual variance
summary(ModelEduc)      # summary prints fixed effects solution 
confint(ModelEduc, level=.95) # confint prints level% CI for fixed effects

txtComment(" ") # insert blank space in external file of saved results


# Center education predictor so that 0 is meaningful
Example2$educ12 = Example2$educ-12
# educ12: Education (0=12 years) # label as a comment only

print("R GLM Predicting Income from Centered Education 0=12 -- save as ModelEduc12")
ModelEduc12 = lm(data=Example2, formula=income~1+educ12)
supernova(ModelEduc12)   # supernova prints residual variance
summary(ModelEduc12)     # summary to print fixed effects solution
confint(ModelEduc12, level=.95) # confint prints level% CI for fixed effects

print("R Get predicted outcomes using glht from multcomp package -- save as PredEduc12")
print("In number lists below, the values are multipliers for each fixed effect in order")
PredEduc12 = glht(model=ModelEduc12, linfct=rbind(
  "Pred Income for  8 years (educ12=-4)" = c(1,-4),  
  "Pred Income for 12 years (educ12= 0)" = c(1, 0), 
  "Pred Income for 16 years (educ12= 4)" = c(1, 4), 
  "Pred Income for 20 years (educ12= 8)" = c(1, 8)))
print("Print glht linear combination results with unadjusted p-values")
summary(PredEduc12, test=adjusted("none")) 
confint(PredEduc12, level=.95, calpha=univariate_calpha())
print ("R standardized fixed effect solution using lm.beta package")
lm.beta(ModelEduc12) 

txtComment(" ") # insert blank space in external file of saved results


####### MODELS WITH BINARY MARITAL STATUS AS A PREDICTOR #######

# Recode marry predictor so that 0 is meaningful
Example2$marry01=NA  # Create new empty variable
Example2$marry01[which(Example2$marry==1)]=0  # marry01=0 if marry=1
Example2$marry01[which(Example2$marry==2)]=1  # marry01=1 if marry=2
# marry01: 0=unmarried, 1=married             # label as a comment only

print("R GLM Predicting Income from Marry01 (0=Unmarried,1=Married) -- save ModelMarry01")
ModelMarry01 = lm(data=Example2, formula=income~1+marry01)
supernova(ModelMarry01)   # supernova prints residual variance
summary(ModelMarry01)     # summary prints fixed effects solution
confint(ModelMarry01, level=.95) # confint to print level% CI for fixed effects

print("R Demonstrating how to get predicted outcomes using glht -- save as PredMarry01")
print("In number lists below, values are multiplier for each fixed effect in order")
PredMarry01 = glht(model=ModelMarry01, linfct=rbind(
  "Pred Income for Unmarried=0" = c(1,0),  
  "Pred income for Married=1"   = c(1,1)))
print("Print glht linear combination results with unadjusted p-values")
summary(PredMarry01, test=adjusted("none")) 
confint(PredMarry01, level=.95, calpha=univariate_calpha())

print("R Compute Cohen's d effect size from t test-statistic manually")
2*6.25/sqrt(732)

print("Compute Cohen's d effect size from t test-statistic using internal values")
as.matrix(summary(ModelMarry01)$coefficients[,3])  # print saved t values
# t test-statistic we want is in row 2 column 1
as.matrix(summary(ModelMarry01)$coefficients[,3])[2,1]*2 / sqrt(ModelMarry01$df.residual)


txtComment(" ") # insert blank space in external file of saved results


################################################################################
#####             BEGIN EXAMPLE 2 BONUS PEARSON CORRELATIONS               #####
################################################################################

if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get bivariate correlations with p-values
# Note: has routine "describe" that competes with psych package below

print("R Pearson Correlation Matrix with P-values using rcorr from Hmisc package")
print("Must convert data frame to matrix to use rcorr")
rcorr(x=as.matrix(Example2[,c("income","educ12","marry01")]), type="pearson")

print("R Pearson Correlation Pairwise Significance tests and CIs")
cor.test(x=Example2$income, y=Example2$educ,  method="pearson", conf.level=.95) 
cor.test(x=Example2$income, y=Example2$marry, method="pearson", conf.level=.95) 
cor.test(x=Example2$educ,   y=Example2$marry, method="pearson", conf.level=.95) 


###############################################################################
# Close output text file
txtStop()

